# Automatically generated by pb2py
# fmt: off
if __debug__:
    try:
        from typing_extensions import Literal  # noqa: F401
    except ImportError:
        pass

ScrambledWords: Literal[0] = 0
Matrix: Literal[1] = 1
