# Automatically generated by pb2py
# fmt: off
if __debug__:
    try:
        from typing_extensions import Literal  # noqa: F401
    except ImportError:
        pass

UnexpectedMessage: Literal[1] = 1
ButtonExpected: Literal[2] = 2
DataError: Literal[3] = 3
ActionCancelled: Literal[4] = 4
PinExpected: Literal[5] = 5
PinCancelled: Literal[6] = 6
PinInvalid: Literal[7] = 7
InvalidSignature: Literal[8] = 8
ProcessError: Literal[9] = 9
NotEnoughFunds: Literal[10] = 10
NotInitialized: Literal[11] = 11
PinMismatch: Literal[12] = 12
WipeCodeMismatch: Literal[13] = 13
InvalidSession: Literal[14] = 14
FirmwareError: Literal[99] = 99
