# Automatically generated by pb2py
# fmt: off
if __debug__:
    try:
        from typing_extensions import Literal  # noqa: F401
    except ImportError:
        pass

TXINPUT: Literal[0] = 0
TXOUTPUT: Literal[1] = 1
TXMETA: Literal[2] = 2
TXFINISHED: Literal[3] = 3
TXEXTRADATA: Literal[4] = 4
TXORIGINPUT: Literal[5] = 5
TXORIGOUTPUT: Literal[6] = 6
