# Automatically generated by pb2py
# fmt: off
if __debug__:
    try:
        from typing_extensions import Literal  # noqa: F401
    except ImportError:
        pass

Strict: Literal[0] = 0
PromptAlways: Literal[1] = 1
PromptTemporarily: Literal[2] = 2
