# Automatically generated by pb2py
# fmt: off
if __debug__:
    try:
        from typing_extensions import Literal  # noqa: F401
    except ImportError:
        pass

Initialize: Literal[0] = 0
Ping: Literal[1] = 1
Success: Literal[2] = 2
Failure: Literal[3] = 3
ChangePin: Literal[4] = 4
WipeDevice: Literal[5] = 5
GetEntropy: Literal[9] = 9
Entropy: Literal[10] = 10
LoadDevice: Literal[13] = 13
ResetDevice: Literal[14] = 14
Features: Literal[17] = 17
PinMatrixRequest: Literal[18] = 18
PinMatrixAck: Literal[19] = 19
Cancel: Literal[20] = 20
LockDevice: Literal[24] = 24
ApplySettings: Literal[25] = 25
ButtonRequest: Literal[26] = 26
ButtonAck: Literal[27] = 27
ApplyFlags: Literal[28] = 28
BackupDevice: Literal[34] = 34
EntropyRequest: Literal[35] = 35
EntropyAck: Literal[36] = 36
PassphraseRequest: Literal[41] = 41
PassphraseAck: Literal[42] = 42
RecoveryDevice: Literal[45] = 45
WordRequest: Literal[46] = 46
WordAck: Literal[47] = 47
GetFeatures: Literal[55] = 55
EndSession: Literal[83] = 83
DoPreauthorized: Literal[84] = 84
PreauthorizedRequest: Literal[85] = 85
Deprecated_PassphraseStateRequest: Literal[77] = 77
Deprecated_PassphraseStateAck: Literal[78] = 78
FirmwareErase: Literal[6] = 6
FirmwareUpload: Literal[7] = 7
FirmwareRequest: Literal[8] = 8
SelfTest: Literal[32] = 32
GetPublicKey: Literal[11] = 11
PublicKey: Literal[12] = 12
SignTx: Literal[15] = 15
TxRequest: Literal[21] = 21
TxAck: Literal[22] = 22
GetAddress: Literal[29] = 29
Address: Literal[30] = 30
SignMessage: Literal[38] = 38
VerifyMessage: Literal[39] = 39
MessageSignature: Literal[40] = 40
GetOwnershipId: Literal[43] = 43
OwnershipId: Literal[44] = 44
GetOwnershipProof: Literal[49] = 49
OwnershipProof: Literal[50] = 50
AuthorizeCoinJoin: Literal[51] = 51
DebugLinkDecision: Literal[100] = 100
DebugLinkGetState: Literal[101] = 101
DebugLinkState: Literal[102] = 102
DebugLinkStop: Literal[103] = 103
DebugLinkMemoryRead: Literal[110] = 110
DebugLinkMemory: Literal[111] = 111
DebugLinkMemoryWrite: Literal[112] = 112
DebugLinkFlashErase: Literal[113] = 113
DebugLinkReseedRandom: Literal[9002] = 9002
DebugLinkRecordScreen: Literal[9003] = 9003
DebugLinkShowText: Literal[9004] = 9004
DebugLinkEraseSdCard: Literal[9005] = 9005
DebugLinkWatchLayout: Literal[9006] = 9006
