# Automatically generated by pb2py
# fmt: off
if __debug__:
    try:
        from typing_extensions import Literal  # noqa: F401
    except ImportError:
        pass

Current: Literal[1] = 1
NewFirst: Literal[2] = 2
NewSecond: Literal[3] = 3
WipeCodeFirst: Literal[4] = 4
WipeCodeSecond: Literal[5] = 5
