# Automatically generated by pb2py
# fmt: off
if __debug__:
    try:
        from typing_extensions import Literal  # noqa: F401
    except ImportError:
        pass

NORMAL: Literal[0] = 0
BOLD: Literal[1] = 1
MONO: Literal[2] = 2
BR: Literal[4] = 4
BR_HALF: Literal[5] = 5
SET_COLOR: Literal[6] = 6
