# Automatically generated by pb2py
# fmt: off
if __debug__:
    try:
        from typing_extensions import Literal  # noqa: F401
    except ImportError:
        pass

Other: Literal[1] = 1
FeeOverThreshold: Literal[2] = 2
ConfirmOutput: Literal[3] = 3
ResetDevice: Literal[4] = 4
ConfirmWord: Literal[5] = 5
WipeDevice: Literal[6] = 6
ProtectCall: Literal[7] = 7
SignTx: Literal[8] = 8
FirmwareCheck: Literal[9] = 9
Address: Literal[10] = 10
PublicKey: Literal[11] = 11
MnemonicWordCount: Literal[12] = 12
MnemonicInput: Literal[13] = 13
_Deprecated_ButtonRequest_PassphraseType: Literal[14] = 14
UnknownDerivationPath: Literal[15] = 15
RecoveryHomepage: Literal[16] = 16
Success: Literal[17] = 17
Warning: Literal[18] = 18
PassphraseEntry: Literal[19] = 19
PinEntry: Literal[20] = 20
