# Automatically generated by pb2py
# fmt: off
if __debug__:
    try:
        from typing_extensions import Literal  # noqa: F401
    except ImportError:
        pass

Bip39: Literal[0] = 0
Slip39_Basic: Literal[1] = 1
Slip39_Advanced: Literal[2] = 2
