# Automatically generated by pb2py
# fmt: off
if __debug__:
    try:
        from typing_extensions import Literal  # noqa: F401
    except ImportError:
        pass

PAYTOADDRESS: Literal[0] = 0
PAYTOSCRIPTHASH: Literal[1] = 1
PAYTOMULTISIG: Literal[2] = 2
PAYTOOPRETURN: Literal[3] = 3
PAYTOWITNESS: Literal[4] = 4
PAYTOP2SHWITNESS: Literal[5] = 5
